.include "asteroid.h.s"
.include "entity.h.s"
.include "spaceship.h.s"
.include "panel.h.s"

.area _DATA
;;=====
;; Private data
;;=====
defineEntity asteroid, #80-asteroid_width-map_margin_right, #50, #-1, #0, #asteroid_width, #asteroid_height, #_pre_asteroid, #1, #15, #0	;; Create asteroid
defineEntity asteroid1, #80-asteroid_width-map_margin_right, #80, #-1, #0, #asteroid_width, #asteroid_height, #_pre_asteroid, #1, #15, #0	;; Create asteroid
defineEntity asteroid2, #80-asteroid_width-map_margin_right, #100, #-1, #0, #asteroid_width, #asteroid_height, #_pre_asteroid, #0, #15, #0	;; Create asteroid

asteroid_timer_create: .db #120 				;; timer to create an asteroid

asteroid_move_timer: .db #15

.area _CODE
;;================;;
;;Public Functions;;
;;================;;

;;=====
;; Update the asteroid
;; input:
;; 		IX=>points to the asteroid data
;; destroys: AF
;;=====
asteroid_update::
	ld a, Ent_a(ix) 							;; A=entity_active
	cp #0 										;; Check if is active
	jr z, not_update 							;; if(active==0) not draw
		ld a, Ent_t(ix)							;; A=asteroid_timer
		cp #0									;; Check if is time to move left
		jr nz, do_not_move_left					;; asteroid_timer != 0

			;;move_left
			ld a, Ent_x(ix)						;; A=asteroid_x
			cp #2+map_margin_left 				;; Check with left limit
			jr z, not_move

				call move_x
				ld a, (asteroid_move_timer)  	;; reset asteroid_timer
				ld Ent_t(ix), a 				;; update asteroid_timer

				ret

			not_move:
				call eraseentity
				call refresh_last_x
				call eraseentity

				call toggleVideoMemory

				call eraseentity
				call refresh_last_x
				call eraseentity

				call toggleVideoMemory

				call decrease_hp
				call draw_hp_simple
				call set_damaged_0

				call set_innactive 				;; asteroid_active=0
			ret

		do_not_move_left:
			dec a 								;; asteroid_timer--
			ld Ent_t(ix), a 					;; update asteroid_timer
	ret
;;		call moveasteroid 						;; move asteroid

	not_update:
ret

;;=====
;; Draw all asteroids
;; destroys: AF
;;=====
asteroid_draw::
	ld a, (asteroid_active)						;; A=asteroid_active
	cp #0 										;; Check if is active
	jr z, not_draw 								;; if(active==0) not draw
		ld ix, #asteroid_data 					;; ix points asteroid_data
		call drawentity							;; Draw our asteroid

	not_draw:	
	
	ld a, (asteroid1_active)					;; A=asteroid1_active
	cp #0 										;; Check if is active
	jr z, not_draw1 							;; if(active==0) not draw
		ld ix, #asteroid1_data 					;; ix points asteroid1_data
		call drawentity							;; Draw our asteroid1

	not_draw1:

	ld a, (asteroid2_active)					;; A=asteroid2_active
	cp #0 										;; Check if is active
	jr z, not_draw2 							;; if(active==0) not draw
		ld ix, #asteroid2_data 					;; ix points asteroid2_data
		call drawentity							;; Draw our asteroid2

	not_draw2:	
ret

;;=====
;; explode all asteroids
;; destroys: IX
;;=====
asteroid_explosion::
	ld a, (asteroid_explosion_stage) 			;; A=asteroid_explosion_stage
    cp #7 										;; check if asteroid is axploding
    jp z, finished 								;; if asteroid_explosion_stage==7, no explode
		ld ix, #asteroid_data					;; IX points to the data
		call draw_explosion						;; explode asteroid
	finished:

	ld a, (asteroid1_explosion_stage) 			;; A=asteroid1_explosion_stage
    cp #7 										;; check if asteroid1 is axploding
    jp z, finished1 							;; if asteroid1_explosion_stage==7, no explode
		ld ix, #asteroid1_data					;; IX points to the data
		call draw_explosion						;; explode asteroid1
	finished1:

	ld a, (asteroid2_explosion_stage) 			;; A=asteroid2_explosion_stage
    cp #7 										;; check if asteroid2is axploding
    jp z, finished2 							;; if asteroid2_explosion_stage==7, no explode
		ld ix, #asteroid2_data					;; IX points to the data
		call draw_explosion						;; explode asteroid
	finished2:
ret

;;=====
;; Gets a pointer to asteroid data in IX
;; destroys: IX
;; returns:
;; 		IX: pointer to asteroid data
;;=====
asteroid_getPtIX::
	ld ix, #asteroid_data						;; IX=asteroid_x
ret

;;=====
;; Gets a pointer to asteroid1 data in IX
;; destroys: IX
;; returns:
;; 		IX: pointer to asteroid1 data
;;=====
asteroid1_getPtIX::
	ld ix, #asteroid1_data						;; IX=asteroid1_x
ret

;;=====
;; Gets a pointer to asteroid2 data in IX
;; destroys: IX
;; returns:
;; 		IX: pointer to asteroid2 data
;;=====
asteroid2_getPtIX::
	ld ix, #asteroid2_data						;; IX=asteroid2_x
ret

;;=====
;; Gets a pointer to asteroid data in IY
;; destroys: IY
;; returns:
;; 		IY: pointer to asteroid data
;;=====
asteroid_getPtIY::
	ld iy, #asteroid_data						;; IY=asteroid_data
ret

;;=====
;; Gets a pointer to asteroid1 data in IY
;; destroys: IY
;; returns:
;; 		IY: pointer to asteroid1 data
;;=====
asteroid1_getPtIY::
	ld iy, #asteroid1_data						;; IY=asteroid1_data
ret

;;=====
;; Gets a pointer to asteroid2 data in IY
;; destroys: IY
;; returns:
;; 		IY: pointer to asteroid2 data
;;=====
asteroid2_getPtIY::
	ld iy, #asteroid2_data						;; IY=asteroid2_data
ret

;;=====
;; Check if there is any asteroid innactive and, if there is, create a new one
;; destroy: AF, IX
;;=====
create_asteroid::
	ld a, (asteroid_timer_create) 				;; A=asteroid_timer_create
	cp #0 										;; Check if it is time to create a new asteroid
	jr nz, not_create_at_all 					;;

		ld a, (asteroid_active)					;; A=asteroid_active
		cp #1 									;; Check if asteroid is active
		jr z, not_create_0 						;; If is active, not create
		ld a, (asteroid_explosion_stage)
		cp #7
		jr nz, not_create_0

			call asteroid_getPtIX 				;; IX=asteroid_data
			call restart_asteroid 				
			call set_active
			ret

		not_create_0:

			ld a, (asteroid1_active)			;; A=asteroid1_active
			cp #1 								;; Check if asteroid1 is active
			jr z, not_create_1 		 			;; If is active, not create
			ld a, (asteroid1_explosion_stage)
			cp #7
			jr nz, not_create_1
				;;Create1
				call asteroid1_getPtIX 			;; IX=asteroid1_data
				call restart_asteroid	
				call set_active 				;; active asteroid
				ret

			not_create_1:
				ld a, (asteroid2_active)		;; A=asteroid2_active
				cp #1 							;; Check if asteroid2 is active
				jr z, not_create_2 		 		;; If is active, not create
				ld a, (asteroid2_explosion_stage)
				cp #7
				jr nz, not_create_2
					;;Create
					call asteroid2_getPtIX 		;; IX=asteroid2_data
					call restart_asteroid
					call set_active 			;; active asteroid2
					ret
				not_create_2:
					ret

				not_create_at_all:
					ld a, (asteroid_timer_create) 	;; A=asteroid_timer_create
					dec a 							;; asteroid_timer_create
					ld (asteroid_timer_create), a 	;; update asteroid_timer_create
ret

;;=====
;; Set values of asteroid to the original values
;; destroys: AF, Ix, HL
;; input:
;; 		IX => asteroid data
;;=====
restart_asteroid::
	call set_innactive 								;; set asteroid innactive
	ld a, #80-asteroid_width-map_margin_right		;; a=original position x of asteroid
	call set_x										;; set position x
	call _getRandomLimitedNumber 					;; get random number for position y of asteroid
	ld a, l
	call set_y 										;; set position y
	call refresh_last_y
	ld a, (asteroid_move_timer) 					;; reset timer to move asteroid
	ld Ent_t(ix), a 								;; update the timer to move asteroid
	ld a, #120 										;; reset timer to create a new asteroid
	ld (asteroid_timer_create), a 					;; update timer to create a new asteroid
ret

;;==================;;
;; Private Functions;;
;;==================;;

;;=====
;; Moves the asteroid
;; input:
;; 		IX => pointer to asteroid data
;; destroy: AF
;;=====

;;moveasteroid:
;;	ld a, Ent_t(ix)									;; A=asteroid_timer
;;	cp #0											;; Check if is time to move left
;;	jr nz, do_not_move_left							;; asteroid_timer != 0
;;
;;		;;move_left
;;		ld a, Ent_x(ix)								;; A=asteroid_x
;;		cp #2+map_margin_left 						;; Check with left limit
;;		jr z, not_move
;;
;;			call move_x
;;			ld a, (asteroid_move_timer)  								;; reset asteroid_timer
;;			ld Ent_t(ix), a 						;; update asteroid_timer
;;
;;			ret
;;
;;		not_move:
;;			call set_innactive 						;; asteroid_active=0
;;			ld a, #80-8-map_margin_right
;;			call set_x 								;; asteroid_x to right limit
;;		ret
;;
;;	do_not_move_left:
;;		dec a 										;; asteroid_timer--
;;		ld Ent_t(ix), a 							;; update asteroid_timer
;;ret

;;========
;; INCREASES THE HORIZONTAL SPEED OF THE ASTEROID
;; INPUT -> A = NEW SPEED
;;========
increase_asteroid_speed:
	ld (asteroid_move_timer), a
ret

;;========
;; INCREASES THE SPAWN SPEED OF THE ASTEROID
;; INPUT -> A = NEW SPEED
;;========
increase_asteroid_spawn:
	ld (asteroid_timer_create), a
ret