.include "entity.h.s"

.area _DATA
.area _CODE
;;=====
;; Draw the entity
;; inputs:
;;      IX => Pointer to entity data
;; destroys: BC, DE , HL, AF
;;=====
drawentity:
	;; Calculate Screen position
	ld a, (buffer_start)
	ld d, a
	ld e, #0x00
	ld c, Ent_x(ix)						;; C=entity x
	ld b, Ent_y(ix)						;; B=entity y
	call cpct_getScreenPtr_asm 			;; Get pointer to screen

	;; Draw a sprite
	ex de, hl 							;; DE points to screen
	ld l, Ent_sprite_l(ix) 				;; HL=sprite pointer
	ld h, Ent_sprite_h(ix) 				;; HL=sprite pointer
	ld c, Ent_w(ix)						;; C=entity w
	ld b, Ent_h(ix)						;; B=entity h
	call cpct_drawSprite_asm
ret

;;=====
;; Erase the entity
;; inputs:
;;      IX => Pointer to entity data
;; destroys: BC, DE , HL, AF
;; ToDo: se puede optimizar haciendo 3 estados de activo
;;=====
eraseentity:
	;; Calculate Screen position
	ld a, (buffer_start)
	ld d, a
	ld e, #0x00
	ld c, Ent_last_x(ix)					;; C=entity x
	ld b, Ent_last_y(ix)					;; B=entity y
	call cpct_getScreenPtr_asm				;; Get pointer to screen

	;; Draw a box
	ex de, hl 								;; DE points to screen
	ld a, #0x00								;; A=backgroundcolor
	ld b, Ent_h(ix) 						;; B=entity height
	ld c, Ent_w(ix) 						;; C=entity width
	call cpct_drawSolidBox_asm
ret

;; Explosion
draw_explosion:
    ;; Calculate Screen position
    ld a, (buffer_start)
    ld d, a
    ld e, #0x00
	ld c, Ent_x(ix)						;; C=entity x
	ld b, Ent_y(ix)						;; B=entity y
    call cpct_getScreenPtr_asm 			;; Get pointer to screen

    ex de, hl
	ld a, Ent_es(ix)
    call draw_explosion_sprite

	ld a, Ent_es(ix)
	inc a
	ld Ent_es(ix), a
ret

restart_explosion:
	ld a, #0
	ld Ent_es(ix), a
ret

;;=====
;; Checks collision between two entitys
;; Input:
;; 		IY => pointer to one entity
;; 		IX => pointer to the other entity
;; Return:
;; 		A=1 if there is a collision
;; 		A=0 if there is not a collision 		
;; destroys: AF, BC
;;=====
entity_checkCollision::
	ld a, Ent_a(iy) 						;; A=entity2_active
	cp #0 									;; Check if entity2 is active
	jr z, no_collision						;; If is not active, no collision
		ld a, Ent_a(ix)						;; A=entity1_active
		cp #0								;; check if entity1 is active
		jr z, no_collision 					;; Check if entity1 is innactive, if it is, dont collide

			;; if(entity1_y + entity1_h <= entity2_y) no collision
			ld a, Ent_y(iy) 				;; A=entity2_y
			ld b, a 						;; B=entity2_y
			ld a, Ent_y(ix)					;; A=entity1_y
			ld c, a 						;; C=entity1_y
			ld a, Ent_h(ix)					;; A=entity1_h
			add c  							;; A=entity1_y+entity1_h
			sub b 							;; A=entity1_y+entity1_h-entity2_y
			jr z, no_collision 				;; if entity1_y+entity1_h-entity2_y==0 no collission
			jp m, no_collision 				;; if entity1_y+entity1_h-entity2_y<0 no collision

			;; if(entity2_y + entity2_h <= entity1_y) no collision
			ld a, Ent_y(ix) 				;; A=entity1_y
			ld c, a 						;; C=entity1_y
			ld a, Ent_h(iy) 				;; A=entity2_h
			ld b, a 						;; B=entity2_h
			ld a, Ent_y(iy) 				;; A=entity2_y
			add b 							;; A=entity2_y+entity2_h
			sub c 							;; entity2_y+entity2_h-entity1_y
			jr z, no_collision 				;; if entity2_y+entity2_h-entity1_y==0 no collission
			jp m, no_collision 				;; if entity2_y+entity2_h-entity1_y<0 no collision

			;;collision 		
			;; if(entity1_x + entity1_w <= entity2_x) no collision
			ld a, Ent_x(iy) 				;; A=entity2_x
			ld b, a 						;; B=entity2_x
			ld a, Ent_x(ix)					;; A=entity1_x
			ld c, a 						;; C=entity1_x
			ld a, Ent_w(ix)					;; A=entity1_w
			add c  							;; A=entity1_x+entity1_w
			sub b 							;; A=entity1_x+entity1_w-entity2_x
			jr z, no_collision 				;; if entity1_x+entity1_w-entity2_x==0 no collission
			jp m, no_collision 				;; if entity1_x+entity1_w-entity2_x<0 no collision

			;; if(entity2_x + entity2_w <= entity1_x) no collision
			ld a, Ent_x(ix) 				;; A=entity1_x
			ld c, a 						;; C=entity1_x
			ld a, Ent_w(iy) 				;; A=entity2_w
			ld b, a 						;; B=entity2_w
			ld a, Ent_x(iy) 				;; A=entity2_x
			add b 							;; A=entity2_x+entity2_w
			sub c 							;; A=entity2_x+entity2_w-entity1_x
			jr z, no_collision 				;; if entity2_x+entity2_w-entity1_x==0 no collission
			jp m, no_collision 				;; if entity2_x+entity2_w-entity1_x<0 no collision

			ld a, #1 						;; A=1
			
			ret

	;;no collision
	no_collision:
		ld a, #0	 						;; A=0
ret

;;=====
;; Change the value of entity_active to 0
;; Input:
;;		ix=> pointer to entity data
;; destroys: AF
;;=====
set_innactive::
	ld a, #0 								;; A=0
	ld Ent_a(ix), a 						;; entity_a=0
ret

;;=====
;; Change the value of entity_active to 1
;; Input:
;;		ix=> pointer to entity data
;; destroys: AF
;;=====
set_active::
	ld a, Ent_a(ix)							;; A=entity_a
	cp #1 									;; Check if the entity is already active
	jr z, not_change_active 				;; if active==1 not change active
		
		ld a, #1 							;; A=1
		ld Ent_a(ix), a 					;; entity_a=1
	
	not_change_active:
ret

;;=====
;; Change the value of entity_x
;; Input:
;; 		A=> position to change entity_x
;;		ix=> pointer to entity data
;;=====
set_x::
	push af									;; save value of a
	ld a, Ent_a(ix) 						;; a=entity_active
	cp #1 									;; Check if the entity is active
	jr z, not_change_x 						;; if a==0 not change
		ld a, Ent_x(ix) 					;; a=entity_x
		ld Ent_last_x(ix), a 				;; entity_last_x=entity_x
		pop af 								;; recover value of a (position for entity_x)
		ld Ent_x(ix), a 					;; entity_x=a
		ret 		
	not_change_x:
		pop af 								;;recover value of af
ret

;;=====
;; Change the value of entity_y
;; Input:
;; 		A=> position to change entity_y
;;		ix=> pointer to entity data
;;=====
set_y::
	push af									;; save value of a
	ld a, Ent_a(ix) 						;; a=entity_active
	cp #1 									;; Check if the entity is active
	jr z, not_change_y 						;; if a==1 not change
		ld a, Ent_y(ix) 					;; entity_y=a
		ld Ent_last_y(ix), a
		pop af 								;; recover value of a (position for entity_y)
		ld Ent_y(ix), a 					;; entity_y=a
		ret 		
	not_change_y:
		pop af 								;;recover value of af
ret

;;=====
;; Change the value of entity_x
;; Input:
;;		ix=> pointer to entity data
;; destroys: AF, BC
;;=====
move_x::
	ld a, Ent_x(ix)							;; Load
	ld Ent_last_x(ix), a					;; Save last
	ld b, a 								;;|
	ld a, Ent_dx(ix)						;;|
	add a, b								;;\Add
	ld Ent_x(ix), a 						;; Update 
ret

;;=====
;; Change the value of entity_y
;; Input:
;;		ix=> pointer to entity data
;; destroys: AF, BC
;;=====
move_y::
	ld a, Ent_y(ix)							;; Load
	ld Ent_last_y(ix), a					;; Save last
	ld b, a 								;;|
	ld a, Ent_dy(ix)						;;|
	add a, b								;;\Add
	ld Ent_y(ix), a 						;; Update 
ret

;;=====
;; Refresh last x position
;; input:
;; 		IX => pointer to entity data
;; destroys: AF
;;=====
refresh_last_x::
	ld a, Ent_x(ix)							;; Load
	ld Ent_last_x(ix), a					;; Save last
ret

;;=====
;; Refresh last y position
;; input:
;; 		IX => pointer to entity data
;; destroys: AF
;;=====
refresh_last_y::
	ld a, Ent_y(ix)							;; Load
	ld Ent_last_y(ix), a					;; Save last
ret