;;=====
;; Includes
;;=====
.include "cpctelera.h.s"
.include "entity.h.s"
.include "inputController.h.s"

.include "intro.h.s"
.include "map.h.s"
.include "panel.h.s"

.include "spaceship.h.s"
.include "enemy.h.s"
.include "asteroid.h.s"
.include "bullet.h.s"

.area _DATA
.globl _initializeRandomGenerators
.globl _song_ingame

keyboard_timer: .db #0x06 				;; timer to scan the keyboard

.area _CODE
;;=====
;; Scan the whole keyboard and music every 6 interrupts
;;=====
isr:
	ex af, af' 							;;|
	exx 								;;|
	push af 							;;|
	push bc 							;;|
	push de 							;;|
	push hl 							;;\save all the registers

	ld a, (keyboard_timer) 				;; A=keyboard_timer
	dec a 								;; keyboard_timer--
	ld (keyboard_timer), a 				;; update keyboard_tiemr
	jr nz, return 						;; if keyboard_timer!=0 return
		call cpct_scanKeyboard_if_asm 	;; scan the keyboard
		call cpct_akp_musicPlay_asm 	;; play music
		call getHp
		cp #1
		jr z, hp_1
			ld a, #6 					;; restart keyboard_timer
			ld (keyboard_timer), a 		;; update keyboard_timer
			jp return
		hp_1:
			ld a, #5 					;; restart keyboard_timer
			ld (keyboard_timer), a 		;; update keyboard_timer
	return:
	pop hl 								;;|
	pop de 								;;|
	pop bc 								;;|
	pop af 								;;|
	exx 								;;|
	ex af, af' 							;;\Recover the registers
ret

;;=====
;;Main program entry
;;=====
_main::
	;; Init Video
    ld sp, #0x7FFF
	call setVideoMode
	call initializeVideoMemory

	;; Set black border
	ld h, #0x14
	call setBorder

	;; Set InterruptHandler
	ld hl, #isr
	call cpct_setInterruptHandler_asm

	;; Initialize music
	ld de, #_song_ingame
	call cpct_akp_musicInit_asm

	;; Intro screen
	call loadTutorialScreen
	call waitKey

	;; Charge the map in memory
	call setMap

	call _initializeRandomGenerators

	ld a, r
	ld l, a
	call cpct_setSeed_lcg_u8_asm

	newgame::
	;;Init Game for BufferC0
	call drawMap 						;; Draw map
	call show_score_text 				;; Show the score text
	call show_hp_text					;; Show the hp text

	call draw_level_text
	call draw_level_simple

	call draw_hp_simple  ;; PA LA FOTO

	call show_logo 						;; show logo

	call toggleVideoMemory				;; change buffer and video memory

	;;Init Game for Buffer80
	call drawMap 						;; Draw map
	call show_score_text 				;; Show the score text
	call show_score 					;; Show the score 
	call show_hp_text					;; Show the hp text

	call draw_level_text
	call draw_level_simple

	call draw_hp_simple 						;; show the hp
	call show_logo 						;; show logo
	

	call toggleVideoMemory				;; change buffer and video memory

	;; Loop
	main_loop:
		;;======================================
		;;ERASE ALL ENTITIES 
		;;======================================
		call spaceship_erase			;;erase the spaceship
		call bullet_erase				;;erase the bullet

		;;======================================
		;; CREATE ENTITIES
		;;======================================
		call create_asteroid 			;;create an asteroid
		call create_enemy 				;;create an enemy

		;;======================================
		;;======================================
		;; UPDATE ENTITIES
		;;======================================
		;;======================================

		;;======================================
		;; SPACESHIP
		;;======================================
		call spaceship_update 			;;update the spaceship

		;;======================================
		;; ENEMIES
		;;======================================
		call enemy_getPtIX				;;ix points enemy_data
		call enemy_getPtIY				;;iy points enemy_data
		call enemy_update 				;;update the enemy

		call enemy1_getPtIX				;;ix points enemy1_data
		call enemy1_getPtIY				;;iy points enemy1_data
		call enemy_update 				;;update the enemy1

		;;======================================
		;; BULLETS 
		;;======================================
		call playerbullet_getPtIX 		;;ix points playerbullet_data
		call bullet_update 				;;update the bullet
		call playerbullet_restart	 	;;check if it is time to restart playerbullet

		call playerbullet1_getPtIX 		;;ix points playerbullet1_data
		call bullet_update 				;;update the bullet
		call playerbullet_restart	 	;;check if it is time to restart playerbullet1

		call playerbullet2_getPtIX 		;;ix points playerbullet2_data
		call bullet_update 				;;update the bullet
		call playerbullet_restart	 	;;check if it is time to restart playerbullet2
		
		call enemybullet_getPtIX 		;;ix points enemybullet_data
		call bullet_update 				;;update the bullet
		call enemybullet_restart 		;;check if it is time to restart enemybullet

		call enemybullet1_getPtIX 		;;ix points enemybullet1_data
		call bullet_update 				;;update the bullet
		call enemybullet_restart 		;;check if it is time to restart enemybullet1

		call enemybullet2_getPtIX 		;;ix points enemybullet2_data
		call bullet_update 				;;update the bullet
		call enemybullet_restart 		;;check if it is time to restart enemybullet2

		;;======================================
		;; ASTEROIDS 
		;;======================================
		call asteroid_getPtIX 			;;ix points asteroid_data
		call asteroid_update			;;update the asteroid

		call asteroid1_getPtIX 			;;ix points asteroid1_data
		call asteroid_update			;;update the asteroid1

		call asteroid2_getPtIX 			;;ix points asteroid2_data
		call asteroid_update			;;update the asteroid2

		;;======================================
		;;======================================
		;; COLLISIONS
		;;======================================
		;;======================================
		;; Cllisions playeerbullet
		;;Check collision between enemy and playerbullet
		call playerbullet_getPtIX 		;; IX=playerbullet_data
		ld a, Ent_a(ix)
		cp #0
		jp z, not_collisionP
			call enemy_getPtIY 			;; IY=enemy_data
			call entity_checkCollision 	;; check colission
			cp #0 						;; 
			jp z, not_collisionEP 		;; if A=0 not collision

				;; Collision
				call increase_score 	;; increase the score
				call set_innactive		;; Deactivate the playerbullet

				call enemy_getPtIX 		;; ix points to enemy_data
				call set_innactive 		;; Deactive the enemy
				call restart_explosion 	;; explode enemy
				call show_score 		;; Shows the score
				jp not_collisionP
			not_collisionEP:

			;;Check collision between enemy1 and playerbullet
			call enemy1_getPtIY 		;; IY=enemy1_data
			call entity_checkCollision 	;; check colission
			cp #0 						;; 
			jr z, not_collisionE1P 		;; if A=0 not collision

				;; Collision
				call increase_score 	;; increase the score
				call set_innactive		;; Desactivate the playerbullet

				call enemy1_getPtIX 	;; ix points to enemy1_data
				call set_innactive 		;; Desactive the enemy
				call restart_explosion
				call show_score 		;; Shows the score ld a, #1
				jp not_collisionP
			not_collisionE1P:

			;;Check collision between asteroid and playerbullet
			call asteroid_getPtIY 		;; IY=asteroid_data
			call entity_checkCollision 	;; check colission
			cp #0 						;; 
			jr z, not_collisionAP 		;; if A=0 not collision

				;; Collision
				call increase_score 	;; increase the score
				call set_innactive		;; Deactivate the playerbullet
			
				call asteroid_getPtIX 	;; ix points to asteroid_data
				call set_innactive 		;; Deactive the asteroid
				call restart_explosion
				call show_score 		;; Shows the score
				jp not_collisionP
			not_collisionAP:

			;;Check collision between asteroid1 and playerbullet
			call asteroid1_getPtIY 		;; IY=asteroid1_data
			call entity_checkCollision 	;; check colission
			cp #0 						;; 
			jr z, not_collisionA1P 		;; if A=0 not collision

				;; Collision
				call increase_score 	;; increase the score
				call set_innactive		;; Desactivate the playerbullet

				call asteroid1_getPtIX 	;; ix points to asteroid1_data
				call set_innactive 		;; Desactive the asteroid1
				call restart_explosion
				call show_score 		;; Shows the score
				jp not_collisionP
			not_collisionA1P:

			;;Check collision between asteroid2 and playerbullet
			call asteroid2_getPtIY 		;; IY=asteroid2_data
			call entity_checkCollision 	;; check colission
			cp #0 						;; 
			jr z, not_collisionA2P 		;; if A=0 not collision

				;; Collision
				call increase_score 	;; increase the score
				call set_innactive		;; Desactivate the playerbullet
				
				call asteroid2_getPtIX 	;; ix points to asteroid2_data
				call set_innactive 		;; Deactive the asteroid2
				call restart_explosion
				call show_score 		;; Shows the score 
				jp not_collisionP
			not_collisionA2P:
		not_collisionP:

		;;Collisions pllayerbullet1
		call playerbullet1_getPtIX 		;; IX=playerbullet1_data
		ld a, Ent_a(ix)
		cp #0
		jp z, not_collisionP1

			;;Check collision between enemy and playerbullet1
			call enemy_getPtIY 				;; IY=enemy_data
			call entity_checkCollision 		;; check colission
			cp #0 							;; 
			jr z, not_collisionEP1 			;; if A=0 not collision

				;; Collision
				call increase_score 		;; increase the score
				call set_innactive			;; Deactivate the playerbullet

				call enemy_getPtIX 			;; ix points to enemy_data
				call set_innactive 			;; Deactive the enemy
				call restart_explosion 		;; explode enemy1
				call show_score 			;; Shows the score
				jp not_collisionP1
			not_collisionEP1:

			;;Check collision between enemy1 and playerbullet1
			call enemy1_getPtIY 			;; IY=enemy1_data
			call entity_checkCollision 		;; check colission
			cp #0 							;; 
			jr z, not_collisionE1P1 		;; if A=0 not collision

				;; Collision
				call increase_score 		;; increase the score
				call set_innactive			;; Deactivate the playerbullet1

				call enemy1_getPtIX 		;; ix points to enemy1_data
				call set_innactive 			;; Deactive the enemy
				call restart_explosion
				call show_score 			;; Shows the score
				jp not_collisionP1
			not_collisionE1P1:

			;;Check collision between asteroid and playerbullet1
			call asteroid_getPtIY 			;; IY=asteroid_data
			call entity_checkCollision 		;; check colission
			cp #0 							;; 
			jr z, not_collisionAP1 			;; if A=0 not collision

				;; Collision
				call increase_score 		;; increase the score
				call set_innactive			;; Desactivate the playerbullet1
				
				call asteroid_getPtIX 		;; ix points to asteroid_data
				call set_innactive 			;; Desactive the asteroid
				call restart_explosion
				call show_score 			;; Shows the score 
				jp not_collisionP1
			not_collisionAP1:

			;;Check collision between asteroid1 and playerbullet1
			call asteroid1_getPtIY 			;; IY=asteroid1_data
			call entity_checkCollision 		;; check colission
			cp #0 							;; 
			jr z, not_collisionA1P1			;; if A=0 not collision

				;; Collision
				call increase_score 		;; increase the score
				call set_innactive			;; Desactivate the playerbullet1

				call asteroid1_getPtIX 		;; ix points to asteroid1_data
				call set_innactive 			;; Desactive the asteroid1
				call restart_explosion
				call show_score 			;; Shows the score 
				jp not_collisionP1
			not_collisionA1P1:

			;;Check collision between asteroid2 and playerbullet1
			call asteroid2_getPtIY 			;; IY=asteroid2_data
			call entity_checkCollision 		;; check colission
			cp #0 							;; 
			jr z, not_collisionA2P1 		;; if A=0 not collision

				;; Collision
				call increase_score 		;; increase the score
				call set_innactive			;; Desactivate the playerbullet1
				
				call asteroid2_getPtIX 		;; ix points to asteroid2_data
				call set_innactive 			;; Deactive the asteroid2
				call restart_explosion
				call show_score 			;; Shows the score
				jp not_collisionP1
			not_collisionA2P1:
		not_collisionP1:

		;; Collisions playerbullet2
		call playerbullet2_getPtIX 		;; IX=playerbullet2_data
		ld a, Ent_a(ix)
		cp #0
		jp z, not_collisionP2

			;;Check collision between enemy and playerbullet2
			call enemy_getPtIY 				;; IY=enemy_data
			call entity_checkCollision 		;; check colission
			cp #0 							;; 
			jr z, not_collisionEP2 			;; if A=0 not collision

				;; Collision
				call increase_score 		;; increase the score
				call set_innactive			;; Desactivate the playerbullet2

				call enemy_getPtIX 			;; ix points to enemy_data
				call set_innactive 			;; Desactive the enemy
				call restart_explosion
				call show_score 			;; Shows the score
				jp not_collisionP2
			not_collisionEP2:

			;;Check collision between enemy1 and playerbullet2
			call enemy1_getPtIY 			;; IY=enemy1_data
			call entity_checkCollision 		;; check colission
			cp #0 							;; 
			jr z, not_collisionE1P2			;; if A=0 not collision

				;; Collision
				call increase_score 		;; increase the score
				call set_innactive			;; Deactivate the playerbullet2

				call enemy1_getPtIX 		;; ix points to enemy1_data
				call set_innactive 			;; Deactive the enemy
				call restart_explosion
				call show_score 			;; Shows the score
				jp not_collisionP2
			not_collisionE1P2:

			;;Check collision between asteroid and playerbullet2
			call asteroid_getPtIY 			;; IY=asteroid_data
			call entity_checkCollision 		;; check colission
			cp #0 							;; 
			jr z, not_collisionAP2 			;; if A=0 not collision

				;; Collision
				call increase_score 		;; increase the score
				call set_innactive			;; Desactivate the playerbullet2
				
				call asteroid_getPtIX 		;; ix points to asteroid_data
				call set_innactive 			;; Desactive the asteroid
				call restart_explosion
				call show_score 			;; Shows the score
				jp not_collisionP2
			not_collisionAP2:

			;;Check collision between asteroid1 and playerbullet2
			call asteroid1_getPtIY 			;; IY=asteroid1_data
			call entity_checkCollision 		;; check colission
			cp #0 							;; 
			jr z, not_collisionA1P2			;; if A=0 not collision

				;; Collision
				call increase_score 		;; increase the score
				call set_innactive			;; Desactivate the playerbullet2

				call asteroid1_getPtIX 		;; ix points to asteroid1_data
				call set_innactive 			;; Desactive the asteroid1
				call restart_explosion
				call show_score 			;; Shows the score
				jp not_collisionP2
			not_collisionA1P2:

			;;Check collision between asteroid2 and playerbullet2
			call asteroid2_getPtIY 			;; IY=asteroid2_data
			call entity_checkCollision 		;; check colission
			cp #0 							;; 
			jr z, not_collisionA2P2			;; if A=0 not collision

				;; Collision
				call increase_score 		;; increase the score
				call set_innactive			;; Deactivate the playerbullet2
				
				call asteroid2_getPtIX 		;; ix points to asteroid2_data
				call set_innactive 			;; Deactive the asteroid2
				call restart_explosion
				call show_score 			;; Shows the score
				jp not_collisionP2
			not_collisionA2P2:
		not_collisionP2:

		;; Collisions spaceship
		call spaceship_getPtIX			;; IX=spaceship_data

		;;Check collision between player and enemybullet
		call enemybullet_getPtIY 		;; IY=enemybullet_data
		call entity_checkCollision 		;; check colission
		cp #0 							;; 
		jr z, not_collisionPEB 			;; if A=0 not collision

			;; Collision
			call enemybullet_getPtIX
			call set_innactive			;; Deactivate the enemybullet
			call decrease_hp 			;; decrease hp
			call draw_hp_simple			;; Shows the score
 			call set_damaged_0 			;; set damaged to 0

		not_collisionPEB:

		;;Check collision between player and enemybullet1
		call enemybullet1_getPtIY 		;; IY=enemybullet1_data
		call entity_checkCollision 		;; check colission
		cp #0 							;; 
		jr z, not_collisionPEB1 			;; if A=0 not collision

			;; Collision
			call enemybullet1_getPtIX
			call set_innactive			;; Deactivate the enemybullet1
			call decrease_hp 			;; decrease hp
			call draw_hp_simple			;; Shows the score 
 			call set_damaged_0			;; set damaged to 0

		not_collisionPEB1:

		;;Check collision between player and enemybullet2
		call enemybullet2_getPtIY 		;; IY=enemybullet2_data
		call entity_checkCollision 		;; check colission
		cp #0 							;; 
		jr z, not_collisionPEB2 			;; if A=0 not collision

			;; Collision
			call enemybullet2_getPtIX
			call set_innactive			;; Deactivate the enemybullet2
			call decrease_hp 			;; decrease hp
			call draw_hp_simple			;; Shows the score 
 			call set_damaged_0 			;; set damaged to 0

		not_collisionPEB2:

		;;Check collision between player and enemy
		call enemy_getPtIY 				;; IY=enemy_data
		call entity_checkCollision 		;; check colission
		cp #0 							;; 
		jr z, not_collisionPE 			;; if A=0 not collision

			;;Collision
			call enemy_getPtIX 			;; ix points to enemy_data
			call set_innactive 			;; Deactive the enemy
			call restart_explosion 		;; explode enemy

			call decrease_hp 			;; decrease hp
			call draw_hp_simple			;; Shows the score 
 			call set_damaged_0 			;; set damaged to 0
		not_collisionPE:

		;;Check collision between player and enemy1
		call enemy1_getPtIY 			;; IY=enemy1_data
		call entity_checkCollision 		;; check colission
		cp #0 							;; 
		jr z, not_collisionPE1 			;; if A=0 not collision

			;;Collision
			call enemy1_getPtIX 		;; ix points to enemy1_data
			call set_innactive 			;; Deactive the enemy1
			call restart_explosion 		;; explode enemy1

			call decrease_hp 			;; decrease hp
			call draw_hp_simple			;; Shows the score 
 			call set_damaged_0 			;; set damaged to 0
		not_collisionPE1:

		;;Check collision between player and asteroid
		call asteroid_getPtIY 			;; IY=asteroid_data
		call entity_checkCollision 		;; check colission
		cp #0 							;; 
		jr z, not_collisionPA 			;; if A=0 not collision

			;;Collision
			call asteroid_getPtIX 		;; ix points to asteroid_data
			call set_innactive 			;; Deactive the asteroid
			call restart_explosion 		;; explode asteroid

			call decrease_hp 			;; decrease hp
			call draw_hp_simple			;; Shows the score 
 			call set_damaged_0 			;; set damaged to 0
		not_collisionPA:

		;;Check collision between player and asteroid1
		call asteroid1_getPtIY 			;; IY=asteroid1_data
		call entity_checkCollision 		;; check colission
		cp #0 							;; 
		jr z, not_collisionPA1 			;; if A=0 not collision

			;;Collision
			call asteroid1_getPtIX 		;; ix points to asteroid1_data
			call set_innactive 			;; Deactive the asteroid1
			call restart_explosion 		;; explode asteroid1

			call decrease_hp 			;; decrease hp
			call draw_hp_simple			;; Shows the score 
 			call set_damaged_0 			;; set damaged to 0
		not_collisionPA1:

		;;Check collision between player and asteroid2
		call asteroid2_getPtIY 			;; IY=asteroid2_data
		call entity_checkCollision 		;; check colission
		cp #0 							;; 
		jr z, not_collisionPA2 			;; if A=0 not collision

			;;Collision
			call asteroid2_getPtIX 		;; ix points to asteroid2_data
			call set_innactive 			;; Deactive the asteroid2
			call restart_explosion 		;; explode asteroid2

			call decrease_hp 			;; decrease hp
			call draw_hp_simple			;; Shows the score 
 			call set_damaged_0 			;; set damaged to 0
		not_collisionPA2:

		;;======================================
		;; Explosions
		;;======================================
		call enemy_explosion
		call asteroid_explosion
		
		;;======================================
		;; DRAW ENTITIES
		;;======================================
		call spaceship_draw				;; draw the spaceship
		call bullet_draw				;; draw the bullet
		call enemy_draw
		call asteroid_draw				;; draw the asteroid2

		call show_lvl_up

		call pressEscape

		call cpct_waitVSYNC_asm
		call toggleVideoMemory			;; change buffer and video memory
		
	jp main_loop