.include "spaceship.h.s"
.include "entity.h.s"

.area _DATA
;;=====
;; Define entity
;;=====
defineEntity spaceship, #map_margin_left+2, #80, #0, #0, #spaceship_width, #spaceship_height, #_pre_ally_spaceship, #1, #0, #0;; Create spaceship

damaged: .db #1
damaged_timer: .db #10 						;;Timer to blend

.area _CODE
;;================;;
;;Public Functions;;
;;================;;
;;=====
;; Update the spaceship
;; destroys: IX
;;=====
spaceship_update:
	ld ix, #spaceship_data					;; IX points to the data
	call checkSpaceshipInput				;; check if user presses keys
ret

;;=====
;; Draw the spaceship
;; destroys: IX, AF
;;=====
spaceship_draw::
	ld ix, #spaceship_data					;; IX points to the data
	ld a, (damaged) 						;; A=damaged
	cp #1 									;; check if the space ship is not damaged
	jr z, draw 								;; if the spaceship is not damaged, draw it normally
		call draw_damaged 					;; if it is damaged, draw it damaged
		ret
	draw:
	call drawentity							;; Draw our spaceship
ret

;;=====
;; Erase the spaceship
;; destroys:  IX
;;=====
spaceship_erase::
	ld ix, #spaceship_data					;; IX points to the data
	call eraseentity						;; Draw our spaceship
ret

;;=====
;; Gets a pointer to spaceship data in IY
;; destroys: IY
;; returns:
;; 		IY: pointer to spaceship data
;;=====
spaceship_getPtIY::
	ld iy, #spaceship_data					;;IY=spaceship_data
ret

;;=====
;; Gets a pointer to spaceship data in IX
;; destroys: IX
;; returns:
;; 		IX: pointer to spaceship data
;;=====
spaceship_getPtIX::
	ld ix, #spaceship_data					;; IX=spaceship_data
ret

;;=====
;;Cast a bullet
;; destroys: AF, IX
;;=====
spaceship_castbullet::
	;; space is pressed
	ld a, (spaceship_timer_shoot) 			;; a=spaceship_timer_shoot
	cp #0 									;; check if is time to shoot
	jr nz, not_shoot 						;; if not, do not shoot
		call playerbullet_getPtIX 			;; IX=playerbullet_data
		ld a, Ent_a(ix) 					;; a=playerbullet_active
		cp #1 								;; check if playerbullet is active
		jr z, not_shoot_0 					;; if playerbullet is active, not shoot this bullet
			ld a, (spaceship_y) 			;; A=spaceship_y
			add a, #8 						;; A+6
			call set_y 						;; set y of playerbullet
			call refresh_last_y
			ld a, #map_margin_left+spaceship_width+1 ;; a=initial position of playerbullet
			call set_x 						;; set playerbullet_x
			call set_active 				;; set playerbullet_active to 1
			ld a, #5 						;; reset timer_shoot
			ld (spaceship_timer_shoot), a 	;;update timer_shoot
			ret
		not_shoot_0:
			call playerbullet1_getPtIX 		;; IX=playerbullet1_data
			ld a, Ent_a(ix) 				;; a=playerbullet1_active
			cp #1 							;; check if playerbullet1 is active
			jr z, not_shoot_1 				;; if playerbullet1 is active, not shoot this bullet
				ld a, (spaceship_y) 		;; A=spaceship_y
				add a, #8 					;; A+6
				call set_y 					;; set y of playerbullet1
				call refresh_last_y
				ld a, #map_margin_left+spaceship_width+1 ;; a=initial position of playerbullet1
				call set_x 					;; set playerbullet1_x
				call set_active 			;; set playerbullet1_active to 1
				ld a, #5 					;; reset timer_shoot
				ld (spaceship_timer_shoot), a 	;;update timer_shoot
				ret
			not_shoot_1:
				call playerbullet2_getPtIX 	;; IX=playerbullet2_data
				ld a, Ent_a(ix) 			;; a=playerbullet2_active
				cp #1 						;; check if playerbullet2 is active
				jr z, not_shoot_2 			;; if playerbullet2 is active, not shoot this bullet
					ld a, (spaceship_y) 	;; A=spaceship_y
					add a, #8 				;; A+6
					call set_y 				;; set y of playerbullet2
					call refresh_last_y
					ld a, #map_margin_left+spaceship_width+1 ;; a=initial position of playerbullet2
					call set_x 				;; set playerbullet2_x
					call set_active 		;; set playerbullet2_active to 1
					ld a, #5 				;; reset timer_shoot
					ld (spaceship_timer_shoot), a 	;;update timer_shoot
					ret
				not_shoot_2:
				ret
	not_shoot:
	dec a 									;; timer_shoot--
	ld (spaceship_timer_shoot), a 			;; update timer_shoot
ret

;;=====
;; Move spaceship up if it is not at the limit
;; destroys: AF
;; input:
;; 		IX => pointer to spaceship data
;;=====
spaceship_moveup::
	ld a, (spaceship_y) 					;; A=spaceship_y
	cp #map_margin_top						;; Check against bottom limit
	jr z, do_not_move_u 					;; spaceship_y != limit, do not move up

		ld a, #-1 							;; A=-1
		ld Ent_dy(ix), a 					;; update direction y of spaceship
		call move_y 						

	do_not_move_u:
ret

;;=====
;; Move spaceship down if he is not at the limit
;; destroys: AF
;; input:
;; 		IX => pointer to spaceship data
;;=====
spaceship_movedown::
	ld a, (spaceship_y) 					;;A=spaceship_y
	cp #450-spaceship_height-map_margin_bottom	;;Check against bottom limit
	jr z, do_not_move_d 					;; spaceship_y != limit, do not move down

		;; move down
		ld a, #1 							;; A=1
		ld Ent_dy(ix), a 					;; update direction y of spaceship
		call move_y

	do_not_move_d:
ret

;;=====
;; Set values of spaceship to the original values
;; destroys: AF, IX
;;=====
restart_spaceship::
	call spaceship_getPtIX 					;; ix points spaceship data
	call set_innactive 						;; set spaceship innactive
	ld a, #80 								;; a=original position y of spaceship
	call set_y 								;; set position y
	ld a, #0 								;; reset timer to shoot of spaceship
	ld Ent_ts(ix), a 						;; update timer to shoot of spaceship
	call set_damaged_1 						;; reset damaged
	call set_active 						;; set spaceship active
ret

;;=====
;; Switch damaged attribute to 1
;; destroys: AF
;;=====
set_damaged_1::
	ld a, #1 								;; A = 1
	ld (damaged), a 						;; update damaged
ret

;;=====
;; Switch damaged attribute to 0
;; destroys: AF
;;=====
set_damaged_0::
	ld a, #0 								;; A = 1
	ld (damaged), a 						;; update damaged
ret

;;=====
;; Draw spaceship damaged
;; destroys: IX, AF, DE, BC, HL
;;=====

draw_damaged::
	call spaceship_getPtIX 					;; IX points spaceship_data
	ld a, (buffer_start) 					
	ld d, a
	ld e, #0x00
	ld c, Ent_x(ix) 						;; C=entity x
	ld b, Ent_y(ix) 						;; B=entity y
	call cpct_getScreenPtr_asm 				;; Get pointer to screen

	;; Draw a sprite
	ex de, hl 								;; DE points to screen
	ld hl, #_pre_damaged_spaceship 			;; HL points to sprite
	ld c, Ent_w(ix) 						;; C=entity w
	ld b, Ent_h(ix) 						;; B=entity h
	call cpct_drawSprite_asm

	ld a, (damaged_timer) 					;; A=damaged_timer
	cp #0 									;; check if damaged_timer is 0
	jr nz, still_damaged 					;; if damaged_timer !=0, still damaged
		call set_damaged_1 					;; set damaged to 1
		ld a, #10 							;; reset damaged_timer
		ld (damaged_timer), a 				;; update damaged_timer
		ret
	still_damaged: 					
	dec a 									;; damaged_timer--
	ld (damaged_timer), a 					;; update damaged_timer
ret